@@ -7,18 +7,11 @@ use warnings;
 use 5.8.0;
 
 package Net::DHCP::Packet;
-$Net::DHCP::Packet::VERSION = '0.7_002';
-# standard module declaration
-our ( @ISA, @EXPORT, @EXPORT_OK );
-use Exporter;
-@ISA       = qw(Exporter);
-@EXPORT    = qw( packinet packinets unpackinet unpackinets ); # FIXME this is rude
-@EXPORT_OK = qw( );
-
-use Socket;
+$Net::DHCP::Packet::VERSION = '0.7_003';
 use Carp;
 use Net::DHCP::Constants qw(:DEFAULT :dhcp_hashes :dhcp_other %DHO_FORMATS);
-use Scalar::Util qw(looks_like_number);    # for numerical testing
+use Net::DHCP::Packet::Attributes qw(:all);
+use Net::DHCP::Packet::IPv4Utils qw(:all);
 use List::Util qw(first);
 
 #=======================================================================
@@ -115,184 +108,6 @@ sub new {
 
 }
 
-#=======================================================================
-# comment attribute : enables transaction number identification
-sub comment {
-    my $self = shift;
-    if (@_) { $self->{comment} = shift }
-    return $self->{comment};
-}
-
-# op attribute
-sub op {
-    my $self = shift;
-    if (@_) { $self->{op} = shift }
-    return $self->{op};
-}
-
-# htype attribute
-sub htype {
-    my $self = shift;
-    if (@_) { $self->{htype} = shift }
-    return $self->{htype};
-}
-
-# hlen attribute
-sub hlen {
-    my $self = shift;
-    if (@_) { $self->{hlen} = shift }
-    if ( $self->{hlen} < 0 ) {
-        carp( 'hlen must not be < 0 (currently ' . $self->{hlen} . ')' );
-        $self->{hlen} = 0;
-    }
-    if ( $self->{hlen} > 16 ) {
-        carp( 'hlen must not be > 16 (currently ' . $self->{hlen} . ')' );
-        $self->{hlen} = 16;
-    }
-    return $self->{hlen};
-}
-
-# hops attribute
-sub hops {
-    my $self = shift;
-    if (@_) { $self->{hops} = shift }
-    return $self->{hops};
-}
-
-# xid attribute
-sub xid {
-    my $self = shift;
-    if (@_) { $self->{xid} = shift }
-    return $self->{xid};
-}
-
-# secs attribute
-sub secs {
-    my $self = shift;
-    if (@_) { $self->{secs} = shift }
-    return $self->{secs};
-}
-
-# flags attribute
-sub flags {
-    my $self = shift;
-    if (@_) { $self->{flags} = shift }
-    return $self->{flags};
-}
-
-# ciaddr attribute
-sub ciaddr {
-    my $self = shift;
-    if (@_) { $self->{ciaddr} = packinet(shift) }
-    return unpackinet( $self->{ciaddr} );
-}
-
-# ciaddr attribute, Raw version
-sub ciaddrRaw {
-    my $self = shift;
-    if (@_) { $self->{ciaddr} = shift }
-    return $self->{ciaddr};
-}
-
-# yiaddr attribute
-sub yiaddr {
-    my $self = shift;
-    if (@_) { $self->{yiaddr} = packinet(shift) }
-    return unpackinet( $self->{yiaddr} );
-}
-
-# yiaddr attribute, Raw version
-sub yiaddrRaw {
-    my $self = shift;
-    if (@_) { $self->{yiaddr} = shift }
-    return $self->{yiaddr};
-}
-
-# siaddr attribute
-sub siaddr {
-    my $self = shift;
-    if (@_) { $self->{siaddr} = packinet(shift) }
-    return unpackinet( $self->{siaddr} );
-}
-
-# siaddr attribute, Raw version
-sub siaddrRaw {
-    my $self = shift;
-    if (@_) { $self->{siaddr} = shift }
-    return $self->{siaddr};
-}
-
-# giaddr attribute
-sub giaddr {
-    my $self = shift;
-    if (@_) { $self->{giaddr} = packinet(shift) }
-    return unpackinet( $self->{giaddr} );
-}
-
-# giaddr attribute, Raw version
-sub giaddrRaw {
-    my $self = shift;
-    if (@_) { $self->{giaddr} = shift }
-    return $self->{giaddr};
-}
-
-# chaddr attribute
-sub chaddr {
-    my $self = shift;
-    if (@_) { $self->{chaddr} = pack( 'H*', shift ) }
-    return unpack( 'H*', $self->{chaddr} );
-}
-
-# chaddr attribute, Raw version
-sub chaddrRaw {
-    my $self = shift;
-    if (@_) { $self->{chaddr} = shift }
-    return $self->{chaddr};
-}
-
-# sname attribute
-sub sname {
-    use bytes;
-    my $self = shift;
-    if (@_) { $self->{sname} = shift }
-    if ( length( $self->{sname} ) > 63 ) {
-        carp( sprintf q|'sname' must not be > 63 bytes, (currently %d)|,
-              length( $self->{sname} ));
-        $self->{sname} = substr( $self->{sname}, 0, 63 );
-    }
-    return $self->{sname};
-}
-
-# file attribute
-sub file {
-    use bytes;
-    my $self = shift;
-    if (@_) { $self->{file} = shift }
-    if ( length( $self->{file} ) > 127 ) {
-        carp( sprintf q|'file' must not be > 127 bytes, (currently %d)|,
-              length( $self->{file} ));
-        $self->{file} = substr( $self->{file}, 0, 127 );
-    }
-    return $self->{file};
-}
-
-# is it DHCP or BOOTP
-#   -> DHCP needs magic cookie and options
-sub isDhcp {
-    my $self = shift;
-    if (@_) { $self->{isDhcp} = shift }
-    return $self->{isDhcp};
-}
-
-# padding attribute
-sub padding {
-    my $self = shift;
-    if (@_) { $self->{padding} = shift }
-    return $self->{padding};
-}
-
-#=======================================================================
-
 sub optionsorder {
 
     # 53 54 55 51 at the start
@@ -369,13 +184,14 @@ sub addOptionValue {
         inets2 => sub { return packinets_array(@_) },
         int    => sub { return pack( 'N', shift ) },
         short  => sub { return pack( 'n', shift ) },
-        byte   => sub { return pack( 'C', 255 & shift ) }
-        ,    # 255 & trims the input to single octet
-        bytes => sub {
+        # 255 & trims the input to single octet
+        byte   => sub { return pack( 'C', 255 & shift ) },
+        bytes  => sub {
             return pack( 'C*', map { 255 & $_ } @_ );
         },
-        string => sub { return shift },
-        csr    => sub { return packcsr(shift) },
+        string     => sub { return shift },
+        clientid   => sub { return packclientid(shift) },
+        csr        => sub { return packcsr(shift) },
         suboptions => sub { return packsuboptions(@_) },
 
     );
@@ -517,7 +333,8 @@ sub getOptionValue {
         byte   => sub { return unpack( 'C', shift ) },
         bytes  => sub { return unpack( 'C*', shift ) },
         string => sub { return shift },
-        csr    => sub { return unpackcsr(shift) },
+        clientid   => sub { return unpackclientid(shift) },
+        csr        => sub { return unpackcsr(shift) },
         suboptions => sub { return unpacksuboptions(shift) },
 
     );
@@ -866,48 +683,6 @@ sub toString {
 
 #=======================================================================
 # internal utility functions
-# never failing versions of the "Socket" module functions
-sub packinet {    # bullet-proof version, never complains
-    use bytes;
-    my $addr = shift;
-
-    if ( $addr && $addr =~ m/(\d+)\.(\d+)\.(\d+)\.(\d+)/ ) {
-        return chr($1) . chr($2) . chr($3) . chr($4);
-    }
-
-    return "\0\0\0\0"
-}
-
-sub unpackinet {    # bullet-proof version, never complains
-    use bytes;
-    my $ip = shift;
-    return '0.0.0.0' unless ( $ip && length($ip) == 4 );
-    return
-        ord( substr( $ip, 0, 1 ) ) . q|.|
-      . ord( substr( $ip, 1, 1 ) ) . q|.|
-      . ord( substr( $ip, 2, 1 ) ) . q|.|
-      . ord( substr( $ip, 3, 1 ) );
-}
-
-sub packinets {     # multiple ip addresses, space delimited
-    return join(
-        q(), map { packinet($_) }
-          split( /[\s\/,;]+/, shift || 0 )
-    );
-}
-
-sub unpackinets {    # multiple ip addresses
-    return join( q| |, map { unpackinet($_) } unpack( '(a4)*', shift || 0 ) );
-}
-
-sub packinets_array {    # multiple ip addresses, space delimited
-    return unless @_;
-    return join( q(), map { packinet($_) } @_ );
-}
-
-sub unpackinets_array {    # multiple ip addresses, returns an array
-    return map { unpackinet($_) } unpack( '(a4)*', shift || 0 );
-}
 
 sub packsuboptions {
     my @relay_opt = @_
@@ -945,6 +720,31 @@ sub unpacksuboptions {     # prints a human readable suboptions
 
 }
 
+sub packclientid {
+   return shift
+   # croak('pack clientid field still WIP');
+}
+
+sub unpackclientid {
+
+    my $clientid = shift
+      or return;
+
+    my $type = unpack('C',substr( $clientid, 0, 1 ));
+
+    if ($type == 0) { # text
+        return substr( $clientid, 1, length($clientid) )
+    }
+    if ($type == 1) { # ethernet
+        return unpack('H*',substr( $clientid, 1, length($clientid) ))
+    }
+
+    return $clientid
+
+   #croak('unpack clientid field still WIP');
+
+}
+
 sub packcsr {
     # catch empty value
     my $results = [ '' ];
@@ -987,18 +787,21 @@ Net::DHCP::Packet - Object methods to create a DHCP packet.
 
 =head1 VERSION
 
-version 0.7_002
+version 0.7_003
 
 =head1 SYNOPSIS
 
    use Net::DHCP::Packet;
 
    my $p = Net::DHCP::Packet->new(
+
         'Chaddr' => '000BCDEF',
         'Xid' => 0x9F0FD,
         'Ciaddr' => '0.0.0.0',
         'Siaddr' => '0.0.0.0',
-        'Hops' => 0);
+        'Hops' => 0
+
+    );
 
 =head1 DESCRIPTION
 
@@ -1090,130 +893,8 @@ Note: DHCP options are created in the same order as key-value pairs.
 
 =head2 ATTRIBUTE METHODS
 
-=over 4
+See L<Net::DHCP::Packet::Attributes>
 
-=item comment( [STRING] )
-
-Sets or gets the comment attribute (object meta-data only)
-
-=item op( [BYTE] )
-
-Sets/gets the I<BOOTP opcode>.
-
-Normal values are:
-
-  BOOTREQUEST()
-  BOOTREPLY()
-
-=item htype( [BYTE] )
-
-Sets/gets the I<hardware address type>.
-
-Common value is: C<HTYPE_ETHER()> (1) = ethernet
-
-=item hlen ( [BYTE] )
-
-Sets/gets the I<hardware address length>. Value must be between C<0> and C<16>.
-
-For most NIC's, the MAC address has 6 bytes.
-
-=item hops ( [BYTE] )
-
-Sets/gets the I<number of hops>.
-
-This field is incremented by each encountered DHCP relay agent.
-
-=item xid ( [INTEGER] )
-
-Sets/gets the 32 bits I<transaction id>.
-
-This field should be a random value set by the DHCP client.
-
-=item secs ( [SHORT] )
-
-Sets/gets the 16 bits I<elapsed boot time> in seconds.
-
-=item flags ( [SHORT] )
-
-Sets/gets the 16 bits I<flags>.
-
-  0x8000 = Broadcast reply requested.
-
-=item ciaddr ( [STRING] )
-
-Sets/gets the I<client IP address>.
-
-IP address is only accepted as a string like '10.24.50.3'.
-
-Note: IP address is internally stored as a 4 bytes binary string.
-See L<Special methods> below.
-
-=item yiaddr ( [STRING] )
-
-Sets/gets the I<your IP address>.
-
-IP address is only accepted as a string like '10.24.50.3'.
-
-Note: IP address is internally stored as a 4 bytes binary string.
-See L<Special methods> below.
-
-=item siaddr ( [STRING] )
-
-Sets/gets the I<next server IP address>.
-
-IP address is only accepted as a string like '10.24.50.3'.
-
-Note: IP address is internally stored as a 4 bytes binary string.
-See L<Special methods> below.
-
-=item giaddr ( [STRING] )
-
-Sets/gets the I<relay agent IP address>.
-
-IP address is only accepted as a string like '10.24.50.3'.
-
-Note: IP address is internally stored as a 4 bytes binary string.
-See L<Special methods> below.
-
-=item chaddr ( [STRING] )
-
-Sets/gets the I<client hardware address>. Its length is given by the C<hlen> attribute.
-
-Value is formatted as an Hexadecimal string representation.
-
-  Example: "0010A706DFFF" for 6 bytes mac address.
-
-Note : internal format is packed bytes string.
-See L<Special methods> below.
-
-=item sname ( [STRING] )
-
-Sets/gets the "server host name". Maximum size is 63 bytes. If greater
-a warning is issued.
-
-=item file ( [STRING] )
-
-Sets/gets the "boot file name". Maximum size is 127 bytes. If greater
-a warning is issued.
-
-=item isDhcp ( [BOOLEAN] )
-
-Sets/gets the I<DHCP cookie>. Returns whether the cookie is valid or not,
-hence whether the packet is DHCP or BOOTP.
-
-Default value is C<1>, valid DHCP cookie.
-
-=item optionsorder ( )
-
-TODO This will rearrange the order of options to accomidate as many
-quirky cliens as possible.
-
-=item padding ( [BYTES] )
-
-Sets/gets the optional padding at the end of the DHCP packet, i.e. after
-DHCP options.
-
-=back
 
 =head2 DHCP OPTIONS METHODS
 
@@ -1316,6 +997,19 @@ This is an empty stub for now
 
 Remove option from option list.
 
+=item I<packclientid( VALUE )>
+
+returns the packed Client-identifier (pass-through currently)
+
+=item I<unpackclientid>
+
+returns the unpackged clientid.
+
+Decodes:
+ type 0 as a string
+ type 1 as a mac address (hex string)
+ everything is passes through
+
 =item I<packcsr( ARRAYREF )>
 
 returns the packed Classless Static Route option built from a list of CIDR style address/mask combos
@@ -1540,75 +1234,24 @@ If the packet is malformed, a fatal error is produced.
 
 Returns a textual representation of the packet, for debugging.
 
-=item packinet ( STRING )
-
-Transforms a IP address "xx.xx.xx.xx" into a packed 4 bytes string.
-
-These are simple never failing versions of inet_ntoa and inet_aton.
-
-=item packinets ( STRING )
-
-Transforms a list of space delimited IP addresses into a packed bytes string.
-
-=item packinets_array( LIST )
-
-Transforms an array (list) of IP addresses into a packed bytes string.
-
 =item packsuboptions ( LIST )
 
 Transforms an list of lists into packed option.
 For option 43 (vendor specific), 82 (relay agent) etc.
 
-=item unpackinet ( STRING )
-
-Transforms a packed bytes IP address into a "xx.xx.xx.xx" string.
-
-=item unpackinets ( STRING )
-
-Transforms a packed bytes list of IP addresses into a list of
-"xx.xx.xx.xx" space delimited string.
-
-=item unpackinets_array ( STRING )
-
-Transforms a packed bytes list of IP addresses into a array of
-"xx.xx.xx.xx" strings.
-
 =item unpacksuboptions ( STRING )
 
 Unpacks sub-options to a list of lists
 
-=back
+=item optionsorder ( )
 
-=head2 SPECIAL METHODS
-
-These methods are provided for performance tuning only. They give access
-to internal data representation , thus avoiding unnecessary type conversion.
-
-=over 4
-
-=item ciaddrRaw ( [STRING])
-
-Sets/gets the I<client IP address> in packed 4 characters binary strings.
-
-=item yiaddrRaw ( [STRING] )
-
-Sets/gets the I<your IP address> in packed 4 characters binary strings.
-
-=item siaddrRaw ( [STRING] )
-
-Sets/gets the I<next server IP address> in packed 4 characters binary strings.
-
-=item giaddrRaw ( [STRING] )
-
-Sets/gets the I<relay agent IP address> in packed 4 characters binary strings.
-
-=item chaddrRaw ( [STRING] )
-
-Sets/gets the I<client hardware address> in packed binary string.
-Its length is given by the C<hlen> attribute.
+TODO This will rearrange the order of options to accomidate as many
+quirky cliens as possible.
 
 =back
 
+See also L<Net::DHCP::Packet::IPv4Utils>
+
 =head1 EXAMPLES
 
 Sending a simple DHCP packet:
